/*
 *  ESC of RC Motor for Cleaner
 *  2022/04/26 ik-gadgets
    Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.81.7
    Device            :  PIC12F1501
    Driver Version    :  2.00
*/

#define MAX_PWM 125 // Pulse width 2msec
#define MIN_PWM 62  // Pulse width 1msec
int pwm_val;
int speed;

int AD_BV;
#define AD_BV_WAR 760 // 電圧低下表示開始 A/D値 (4.96V * AD_BV_WAR /1023*2 ≒ 7.37V)
#define AD_BV_MIN 660 // 過放電保護 A/D値 (4.96V * AD_BV_WAR /1023*2 ≒ 6.40V)
int LED_CNT = 0; // 電圧低下時LED点滅表示用

#include "mcc_generated_files/mcc.h"

void proc_ESC_init(void){
    __delay_ms(100);
    PWM3_LoadDutyValue(MIN_PWM);
    __delay_ms(3000);
    PWM3_LoadDutyValue(MAX_PWM);
    __delay_ms(1000);
    PWM3_LoadDutyValue(MIN_PWM);
}

void main(void)
{
    // initialize the device
    SYSTEM_Initialize();

    // When using interrupts, you need to set the Global and Peripheral Interrupt Enable bits
    // Use the following macros to:

    // Enable the Global Interrupts
    //INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptEnable();

    // Disable the Global Interrupts
    // INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    // INTERRUPT_PeripheralInterruptDisable();

    proc_ESC_init();
    
    while (1)
    {
        if(IN_PORT){
            speed = MIN_PWM;
        }
        else{
            speed = MIN_PWM + ADC_GetConversion(VR)/16;
            if (speed < MIN_PWM) speed = MIN_PWM;
            else if (speed > MAX_PWM) speed = MAX_PWM;
        }
        
        // check Battery Voltage //
        AD_BV = ADC_GetConversion(BV);
        if (AD_BV <= AD_BV_MIN){
            speed = MIN_PWM;
            pwm_val = speed;
        }
        else if (AD_BV <= AD_BV_WAR){
            if (LED_CNT>0){
                LED_CNT-= 2;
                if (LED_CNT <= (AD_BV - AD_BV_MIN)) LED_SetHigh();
            }
            else{
                LED_CNT = 100;
                LED_SetLow();
            }
        }
        else{
            LED_SetHigh();
        }
        
        if (pwm_val < speed) pwm_val++;
         else if (pwm_val > speed) pwm_val--;
        PWM3_LoadDutyValue(pwm_val);
        
        __delay_ms(10);
    }
}
/**
 End of File
*/